;////////// UM6576 Symbol define for both UM6561 mode and UM6578 mode \\\\\\\\\\
;
;======== Section 1. PPU Control register define for UM6561 mode ========
;
SystemControl		.equ	$2000		; UM6576 System Control Register :
						;	D0, D1 : Page selection of background graphics for
						;		   screen, Thease two bits set display pages.
						;			0 0 - Page 0 (Up and Left side page).
						;			0 1 - Page 1 (Up and Right side page).
						;			1 0 - Page 2 (Low and Left side page).
						;			1 1 - Page 3 (Low and Right side page).
						;
						;	D2 : Increment amount of address counter during
						;	        Read or Write data into VRAM, this function
						;	        refer to port $2006 and $2007 register.
						;		D2 = 0, VRAM increment by 1 byte for
						;			 horizontal sequence;
						;		D2 = 1, VRAM increment by 32 bytes for
						;			 vertical sequence;
						;
						;	D3 : A12 of VRAM page select for sprite pattern.
						;
						;	D4 : A12 of VRAM page select for background
						;	        graphic pattern.
						;
						;	D5 : Pattern size select sprite.
						;	 	D5 = 0, 8 X 8 pixels for each pattern;
						;		D5 = 1, 8 X 16 pixels for each pattern;
						;
						;	D6 : Reseved bit.
						;
						;	D7 : NMI request control register.
						;		D7 = 0, Disable the NMI requeset during
						;			 Vertical Blank;
						;		D7 = 1, Enable the NMI request during
						;			 Vertical Blank;

VideoControl		.equ	$2001		; Video ouput function control register :
						;	D0 : Video output control.
						;		D0 = 0, Color output mode;
						;		D0 = 1, Black and White output mode;
						;
						;	D1 : Horizontal resulotion control for background.
						;		D1 = 0, Horizontal resulotion is 248, the
						;			 range start from X=8 to X=255;
						;		D1 = 1, Horizontal resulotion is 256, the
						;			 range start from X=0 to X=255;
						;
						;	D2 : Sprite display range control.
						;		D2 = 0, ?
						;		D2 = 1, No range limited;
						;
						;	D3 : Background graphic display control.
						;		D3 = 0, Disable background display;
						;		D3 = 1, Enable background display;
						;
						;	D4 : Sprite display control.
						;		D4 = 0, Disable sprite display;
						;		D4 = 1, Enable sprite display;
						;
						;	D5 : Blue color output control.
						;		D5 = 0, Blue color enhance output;
						;		D5 = 1, Blue color normal output;
						;
						;	D6 : Green color output control.
						;		D6 = 0, Blue color enhance output;
						;		D6 = 1, Blue color normal output;
						;
						;	D7 : Red color output control.
						;		D7 = 0, Blue color enhance output;
						;		D7 = 1, Blue color normal output;
						;

PPUStatus		.equ	$2002		; PPU status register :
						;	D0~D4 : Reseved bits;
						;	D5 : Over 8  sprites on same horizontal line flag.
						;		D5 = 0, Not over 8 sprites on any line;
						;		D5 = 1, Over 8 sprites on same line;
						;
						;	D6 : Pattern 1 of sprite collides with background
						;	        pattern.
						;		D6 = 0, No collides happen;
						;		D6 = 1, Sprite pattern collides with
						;			 background pattern;
						;
						;	D7 : NMI flag.
						;		D7 = 0, PPU Not during Vertical balnk;
						;		D7 = 1, PPU just during Vertical blank;
						;

SpriteStartAddress	.equ	$2003		; Preset Sprite data start adress for Sprite data string, because
						; total 256 bytes memory on the VRAM as sprite buffer, who is
						; used for store total 64 sprites data, and user can change the
						; start address before transfer sprite data into Sprite buffer by
						; SpriteDMA, change start address of sprite buffer will cause
						; the sprite priority change during displaying.

SpriteDataPort		.equ	$2004		; Entry port for transfer sprite data into Sprite Buffer in VRAM,
						; this port function as  a interface port between CPU and sprite
						; buffer. To use this port must transfer 4 bytes data as a group
						; by sequency store command or Sprite DMA, The 4 bytes
						; data spec as follow:
						;	Byte 0 : Vertical position for Sprite;
						;	Byte 1 : PGT index number;
						;	Byte 2 : Sprite Control parameter
						;			D7 - Vertical mirror control bit;
						;			D6 - Horizontal mirror control bit;
						;			D5 - Priority Control bit;
						;			D4, D3, D2 - Unused bits;
						;			D1, D0 - Color pallete number;
						;	Byte 4 : Horizontal position for Sprite;

ScreenPosition		.equ	$2005		; Display origin point setup register.
						; This register is used for setting the display area of
						; background graphics. The software must write 2
						; bytes data into this register for setup the X, Y position,
						; first byte for X position then second byte for Y.

VRAMAddress	.equ	$2006		; VRAM start address setup register.
						; This register is used for setting the start address of VRAM
						; or background pattern ROM. This register need 2 bytes
						; data for a complete address(16 bits), first byte data set to
						; 'GA15~GA8', then second byte set to 'GA7~GA0'.

VRAMData		.equ	$2007		; VRAM data read or write register.
						; The CPU read or write data to VRAM must though
						; this register. After setup the VRAM address from $2006h,
						; the CPU can read or write into VRAM from $2007. After
						; each read or write action, the VRAM address counter will
						; increase by 1 or 32 automaticlly(refer to $2000h).
						; Note : Each time try to read the VRAM data after set
						;	 address into $2006h or after write data into $2007,
						;	 the first read action will turn mode of $2007 to read
						;	 mode, so the first read action will be inhibited, then
						;	 second read action will get first VRAM data which
						;	 are you want and other read action will be normal.

;
;======== Section 2. Extend PPU Control register define for UM6578 mode ========
;
ColorSelect		.equ	$2008		; Color select and PNT start address setup register.
						;	D0 : Start address of PNT table for background
						;	        graphics ;
						;		The value of this bit will set to GA13,
						;		 and GA14 and GA15 will set to 0 during
						;		 PPU access to background graphics data.
						;
						;	D1 : Reseved bit; 
						;
						;	D2, D3 : Start address of PGT table for Sprite;
						;		The value of those bits will set to GA12 and
						;		 GA13, and GA14 and GA15 will set to 0
						;		 during PPU access the Sprite data.
						;
						;	D4~D6 : Reseved bits;
						;
						;	D7 : Color mode selection for background graphics;
						;		D7 = 0, 4 Colors mode for each cell and
						;			 font, the function same as UM6561;
						;		D7 = 1, 16 Colors mode for each font, and
						;			 delete the 'cell' parameter;
;
;======== Section 3. PSG Control register define for UM6561 mode ========
;
SpriteDMASource	.equ	$4014		; Sprite DMA source address setup register for Sprite data.
						; D0~D7 : The contain value defines the high byte of CPU
						; 	     address(A15~A8).
						; This DMA is used for Sprite data only, and the only way 
						; to renew sprite's data in VRAM is by this DMA. Writing 
						; this register will start DMA immeditedly. The destination
						; address of Sprite DMA always set to Sprite Data Port($2004);
						;

PSGIRQstatus		.equ	$4015		; PSG sound generator enable control(write mode) and PSG
						; channel status with IRQ status flag(read mode).
						;     Write mode -
						;	D0 : Channel 1 enable control of PSG;
						;	D1 : Channel 2 enable control of PSG;
						;	D2 : Channel 3 enable control of PSG;
						;	D3 : Channel 4 enable control of PSG;
						;	D4 : Channel 5 enable control of PSG;
						;     Read mode -
						;	D0 : Channel 1 active status;
						;	D1 : Channel 2 active status;
						;	D2 : Channel 3 active status;
						;	D3 : Channel 4 active status;
						;	D4 : Channel 5 active status;
						;	D5 : Reseved bit;
						;	D6 : IRQ status flag;
						;	D7 : Channel 5 IRQ flag;
						;

StrobeControl		.equ	$4016		; I/O Strobe signal control register.
						;	D0 : Joystick preset control signal(write mode) and
						;	        Joystick 1 serial data input port(read mode).
						;	        Write Mode -
						;		D0 = 0, Enable Joystick preset signal for
						;			 load parallel data into serial shift
						;			 register in joystick 1 and joystick 2;
						;		D0 = 1, Disable Joystick preset signal for set	
						;			 the serial shift register change to
						;			 serial transfer status, refer to read
						;			 mode of $4016 or $4017 for read
						;			 back joystick data;
						;	        Read Mode -
						;		D0 is the serial in data from Joystick 1, read
						;		$4016 port each time will generate a shift
						;		clock pulse to joystick 1 automatically, so if
						;		want to read 8 bits data back, must read this
						;		port 8 times;
						;
						;	D1 : Extend enable signal 2 for control user defined
						;	        external I/O port.
						;		D1 = 0, Enable external I/O port;
						;		D1 = 1, Disable external I/O port;
						;
						;	D2 : Extend enable signal 1 for control user defined
						;	        external I/O port.
						;		D2 = 0, Enable external I/O port;
						;		D2 = 1, Disable external I/O port;
						;
						;	D3~D7 : Reseved bits;
						;

Joystick2Control	.equ	$4017		; Joystick 2 serial data input port(read mode) with IRQ control
						; register(write mode). 
						;	D0 : Joystick 2 serial data input port.
						;		D0 is the serial in data from Joystick 2, read
						;		$4017 port each time will generate a shift
						;		clock pulse to joystick 2 automatically, so if
						;		want to read 8 bits data back, must read this
						;		port 8 times;
						;	Note : D0 is actived during read mode only.
						;
						;	D1~D5 : Reseved bits;
						;
						;	D6 : Mark sequential IRQ;
						;
						;	D7 : Mark sequential Irq with change envelope base
						;	        frequency;
						;	Note : D6, D7 is actived during write mode only.
						;

;
;======== Section 4. Extend PSG Control register define for UM6578 mode ========
;
KeyboardDataIn	.equ	$4020		; Keyboard data input port for read data from PC/AT keyboard.

KeyboardDataOut	.equ	$4021		; Keyboard data output port for write data to PC/AT keyboard.

KeyboardControl	.equ	$4022		; Keyboard communication direction control.
						;	D7 : Keyboard colock control;
						;		D7 = 0, Keyboard inhibit and UM6576 repare
						;			 send data to keyboard;
						;		D7 = 1, UM6576 ready to recive data from 
						;			 keyboard;
						;
						;	D6-D1 : Reseved bits;
						;
						;	D0 : Communication direction control;
						;		D0 = 0, Recive mode - Keyboard to UM6576;
						;		D0 = 1, Transmit mode - UM6576 to Keyboard;
						;
						;	Note : User must set D0=0 and D7=1 for ready to recive
						;	           data from Keyboard during Recive mode.
						;

MouseControl		.equ	$4023		; Joystick or Mouse for Port 2 selection control.
						;	D0 : Joystick/Mouse selection;
						;		D0 = 0, set Joystick connect on Port 2;
						;		D0 = 1, set Mouse connect on Port 2;
						;
						;	D1~D7 : Reseved bits;
						;

MouseDataIn		.equ	$4024		; Data port for Mouse if Mouse connected on Port 2.
						;	D0~D7 : Mouse data;
						;

ExternalIO		.equ	$4026		; External I/O port for application control.
						;	D0 : Chip Select for Real Time Clock;
						;
						;	D1 : Reset signal for Mouse;
						;		D1 = 0, Normal status;
						;		D1 = 1, Reset signal for mouse;
						;
						;	D2~D7 : Reseved bits for application by user;
						;

PCMData		.equ	$4027		; Data register for 8 bits D/A converter.
						;	D0~D7 : PCM data;
						;

SystemIRQMask	.equ	$4032		; Mask and clear control register for system interrupt.
						;	D7 : Timer Interrupt;
						;		D7 = 0, Enable Timer Interrupt;
						;		D7 = 1, Disable and clear Timer Interrupt;
						;
						;	D6 : Mouse Data Interrupt;
						;		D6 = 0, Enable Mouse Data Interrupt;
						;		D6 = 1, Disable and clear Mouse Data Interrupt;
						;
						;	D5 : Keyboard Data Interrupt;
						;		D5 = 0, Enable Keyboard Data Interrupt;
						;		D5 = 1, Disable and clear Keyboard Data Interrupt;
						;
						;	D4 : Real Time Colok Interrupt;
						;		D4 = 0, Enable R.T.C. Interrupt;
						;		D4 = 1, Disable and clear R.T.C. Interrupt;
						;
						;	D3 : Floppy Disk Driver Data Interrupt;
						;		D3 = 0, Enable F.D.D. Data Interrupt;
						;		D3 = 1, Disable and clear F.D.D. Data Interrupt;
						;
						;	D2 : Printer Data Interrupt;
						;		D2 = 0, Enable Printer Data Interrupt;
						;		D2 = 1, Disable and clear Printer Data Interrupt;
						;
						;	D1 : Phi0 Interrupt;
						;		D1 = 0, Enable Phi0 Interrupt;
						;		D1 = 1, Disable and clear Phi0 Interrupt;
						;
						;	D0 : Reseved bit;
						;

SystemIRQStatus	.equ	$4033		; Status register for system interrupt.
						;	D7 : Timer Interrupt;
						;		D7 = 0, Timer Interrupt not occurs;
						;		D7 = 1, Timer Interrupt occurs;
						;
						;	D6 : Mouse Data Interrupt;
						;		D6 = 0, Mouse Data not occurs;
						;		D6 = 1, Mouse Data occurs and ready for read;
						;
						;	D5 : Keyboard Data Interrupt;
						;		D5 = 0, Keyboard Data not occurs;
						;		D5 = 1, Keyboard Data occurs and ready for read;
						;
						;	D4 : Real Time Colok Interrupt;
						;		D4 = 0, R.T.C. Interrupt not occurs;
						;		D4 = 1, R.T.C. Interrupt occurs;
						;
						;	D3 : Floppy Disk Driver Data Interrupt;
						;		D3 = 0, F.D.D. Data  Interrupt not occurs;
						;		D3 = 1, F.D.D. Data Interrupt occurs;
						;
						;	D2 : Printer Data Interrupt;
						;		D2 = 0, Printer Data Interrupt not occurs;
						;		D2 = 1, Printer Data Interrupt occurs;
						;
						;	D1 : Phi0 Interrupt;
						;		D1 = 0, Phi0 Interrupt not occurs;
						;		D1 = 1, Phi0 Interrupt occurs;
						;
						;	D0 : Reseved bit;
						;

SystemTimerControl	.equ	$4034		; System Timer Control register.
						;	D7 : Timer enable control;
						;		D7 = 0, Disable Timer Interrupt;
						;		D7 = 1, Enable Timer Interrupt;
						;
						;	D6 : Timer count value autoload control;
						;		D6 = 0, No-autoload, count one time per trigger.
						;		D6 = 1, Autoload value after interrupt occurrs and
						;			 countinues counting;
						;
						;	D5 : Base clock source selection;
						;		D5 = 0, Use system clock for clock source
						;			 (1.7898MHz);
						;		D5 = 1, Use H-Blank for clock source(15625Hz);
						;
						;	D4 : Reseved bit;
						;
						;	D3-D0 : Prescale value;

SetTimerValue		.equ	$4035		; Timer count value setup register.
						;	D7-D0 : Count value;

ReadTimerValue	.equ	$4036		; Current Timer count value register for read.
						;	D7-D0 : Current counter value;

SpecialControl		.equ	$4020		; Enable control register for special function(Write Only).
						;	D7 : Enable/Disable control;
						;		D7 = 0, System DMA function disable, only VRAM
						;			 DMA can be work(Mode 0);
						;		D7 = 1, System DMA function enable, all of the
						;			  system DMA can be work, include Mode 0
						;			  to Mode 3;
						;
						;	D6 : Reseved bits;
						;
						;	D5 : Keyboard/PS2 Mouse Clock trigger control;
						;		D5 = 0, set trigger at rising edge of Clock single;
						;		D5 = 1, set trigger at falling edge of clock single;
						;
						;	D4-D0 : Reseved bits;
						;

DMAControl		.equ	$4048		; System DMA control register.
						;	D7 : DMA Enable control(Write mode) or DMA status
						;	        flag(Read mode);
						;	        Write mode -
						;		D7 = 0, System DMA disable;
						;		D7 = 1, System DMA enable, and DMA wil
						;			 trigged by V-Blanking automatically,
						;			 or start immediately if the PPU is Off.
						;
						;	D6, D5 : DMA mode selection;
						;		0 0 - Mode 0, for double cycle to DMA data from
						;			          source of system bus to VRAM;
						;		1 0 - Mode 1, for single cycle to DMA data from
						;			          source of system bus  to VRAM;
						;		0 1 - Mode 2, for double cycle to DMA data from
						;			          source of system bus to destination
						;			          of system bus;
						;		1 1 - Mode 3, same as Mode 2;
						;
						;	D4~D0 : Reseved bits;
						;

DMASourceBank	.equ	$4049		; Bank select register for DMA source.
						;	D7~D5 : Reseved bits;
						;	D4~D0 : mapping to XA20~XA16 of system bus;

DMASourceL		.equ	$404A		; Low byte address register of DMA Source.
						;	D7~D0 : mapping to A7~A0 of system bus;

DMASourceH		.equ	$404B		; High byte address register of DMA Source.
						;	D7~D0 : mapping to A15~A8 of system bus;

DMADestinationL	.equ	$404C		; Low byte address register of DMA Destination.
						;	D7~D0 : mapping to A7~A0 of system bus or
						;		  GA7~GA0 of VRAM bus;

DMADestinationH	.equ	$404D		; High byte address register of DMA Destination.
						;	D7~D0 : mapping to A15~A8 of system bus or
						;		  GA15~GA8 of VRAM bus;

DMALengthL		.equ	$404E		; Low byte register of DMA length counter.
						;	D7~D0 : mapping to L7~L0 of length counter;

DMALengthH		.equ	$404F		; High byte register of DMA length counter.
						;	D7 : Reseved bit;
						;	D6~D0 : mapping to L14~L8 of length counter;

;
;======== Section 5. I/O Peripheral Control register define for UM6561 mode ========
;

;
;======== Section 6. Extend I/O Peripheral Control register define for LCD testing module. ========
;
LCDControl		.equ	$4400		; Control register for LCD module control signal.
						;	D7 : LCD Module R/W signal.
						;		D7 = 0, Write data into LCD module;
						;		D7 = 1, Read data from LCD module;
						;
						;	D6 : LCD Enable control signal.
						;		D6 = 0, LCD can not accept any instruction;
						;		D6 = 1, LCD can accept instruction but must work
						;			 with other signals;
						;
						;	D5 : LCD RS control signal.
						;		D5 = 0, LCD Instruction data Input or Output;
						;		D5 = 1, Display data Input or Output;
						;
						;	D4~D0 : Reseverd Bits.
						;

LCDData		.equ	$4480		; Data I/O port for LCD Module.

CPUSpeedSet		.equ	$FFDA		; CPU speed selection register for PAL-B TV system.
						; After system reset, the system clock for CPU is Xtal/15,
						; user can write data($FEh) to this port for switch system
						; clock slow down to Xtal/16 for meet PAL-B TV system,
						; and change the waveform of PHI2 signal from CPU, which
						; will change to 50%-50% duty circle.
;
;======== Section 6. Extend I/O Peripheral Control register define for UM6578 mode ========
;
